/*
 * This code was created by Jeff Molofee '99 
 * (ported to Linux/SDL by Ti Leggett '01)
 *
 * If you've found this code useful, please let me know.
 *
 * Visit Jeff at http://nehe.gamedev.net/
 * 
 * or for port-specific comments, questions, bugreports etc. 
 * email to leggett@eecs.tulane.edu
 */
 
 /*
  * Used as base code for 3d lockpicking game by JKtheCJer
  * Do not try to understand the source; that's impossible.
  * Instead, try to realize the truth...there is no source.
  * Then you will find it is not the app that is modded;
  * It is only yourself
  * --------------------------------------------------------
  * SOURCE CODE NOTES:
  * I mostly reject any and all good programming practices.
  * Commenting is sparse and mostly useless.
  * However if you have a question, I may be able to answer it.
  * jkthecjer@gmail.com
  *
  * I have a habit of using counter variables starting from c.
  * So instead of the usual i, j, k, etc..; you'll find c, d, etc..
  * Most variables with silly name like "crap" and "bullshit" are used for tests
  * Example: a loop excutes, looking for a condition, if it finds it, it sets
  *   "bullshit" to 1.  After the loop has executed, it check for "bullshit" and
  *   acts accordingly.  I try only to use this technique if a break would not 
  *   do the job.
  *--------------------------------------------------------
  * And as always: remember...patience :-)
  *--------------------------------------------------------
  *Build by typing make
  *Run by typing ./lock
  */
 
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include <string.h>
#include "SDL.h"

/* screen width, height, and bit depth */
#define SCREEN_WIDTH  800
#define SCREEN_HEIGHT 600
#define SCREEN_BPP     16

/* Define our booleans */
#define TRUE  1
#define FALSE 0


struct jk_vertex {
	

	float x;
	float y;
	float z;
};

struct jk_face {
	
	int type;	

	int v1;
	int v2;
	int v3;	
    int v4;
	
	int fn;

	float red;
	float green;
	float blue;
};

struct jk_object {
	int f[500000];
	int maxF;
	
	float transX;
	float transY;
	float transZ;

	float scale;

	float boundlow;
	float boundhigh;
	float boundleft;
	float boundright;
	int gravity;
	
	int firstV;
	int numV;
	
	int invisible;
	
	char * name;
};

struct jk_pin {
	char * name;
	
	float notchStart[10];
	float notchEnd[10];
	float notchDepth[10];
	
};

struct jk_lock {
	char * name;
	
	float tolerance;
	int numPins;
	
	int topin[7];
	float pinLen[7];
	
	float fastTimeFull;
	float fastTimePlug;
	float fastTimeRealistic;
	
};

/* This is our SDL surface */
SDL_Surface *surface;

const float GRAVITY = 0.05f;

float cameraX = 20.0f;
float cameraY = 0.0f;
float cameraZ = 8.0f;

float cameraRotX = 0.0f;
float cameraRotY = 305.0f;
float cameraRotZ = 0.0f;

int keyUp = 0;
int keyDown = 0;
int keyLeft = 0;
int keyRight = 0;
int keyA = 0;
int keyS = 0;
int keyD = 0;
int keyF = 0;
int keyG = 0;
int keyZ = 0;
int keyX = 0;
int keyC = 0;
int keySpace = 0;

float PIN_SPACING = 0.5f;

float pinPush[10];
float pinPushing[10];
int pinPushingContour[10];

float shearline = -0.44f;

int maxContours = 0;

float pinSet[10];
float pinOverset[10];
int pinNotchSet[10];
int falseSet[10];
float pinLen[10];

int pinOrderPerm[10];
int pinOrder[10];
int pinType[10];

float pinWidth = 0.2f;

float tension = 0.0f;
float maxTension = 3.0f;
float minTension = 1.0f;
float overTension = 2.5f;
float pushForce = 0.01f;

float tensionRate = 0.01f;
float tensionLoss = 0.005f;

float pinResistance = 0.005;
float pinSetResistance = 0.05;
float pinOversetResistance = 0.01;

float setTension = 0.2;

int currPick = 26;

float plugPlay = 3;
float setTolerance = 0.01;

int setResistance = 1;
int oversetResistance = 20;
int notchCrap = 20;
float setRot = 0.4f;

float pickOffset = -0.65f;

GLint timeStart;
GLint timeEnd;
int timeOn;

int startSprings, startTop, startPins;



const float piover180 = 0.0174532925f;

GLint t;

int maxQuads;
int totalV;
int totalVn;
int totalObj;

struct jk_vertex pickContour[100];
struct jk_vertex v[5000000];
struct jk_vertex vn[5000000];
struct jk_face face[500000];
struct jk_object obj[100];
struct jk_pin pin[10];

struct jk_lock lock[50];

float plugRot = 180.0f;

float pickAngle = 0.0f;

int mode = 2;
int clicked = 0;

int currMenu = 0;
int menuSel = 0;

int currLock = 0;
char * visibility = "Full";

GLuint  base; /* Base Display List For The Font Set */
GLfloat cnt1; /* 1st Counter Used To Move Text & For Coloring */
GLfloat cnt2; /* 2nd Counter Used To Move Text & For Coloring */

XFontStruct *fontInfo;
XFontStruct *fontInfo2;
Display *dpy; 

void loadLock(int index);

void rotate(float a)
{
	
	if (pickAngle <= -30.0 && a < 0)
		return;
	
	if (pickAngle >= 30.0 && a > 0)
		return;
	
	
	long c = 0;
	float Xnew, Xold, Ynew, Yold;
	
	float a2 = a;
	
	a = a * piover180;
	
	for(c = obj[currPick].firstV; c < obj[currPick].numV; c++)
	{
		Xold = v[c].x;
		Yold = v[c].y;
		Xnew = cos(a) * Xold - sin(a) * Yold;
		Ynew = sin(a) * Xold + cos(a) * Yold;
		v[c].x = Xnew;
		v[c].y = Ynew;		
	}
	
	for(c = obj[currPick + 1].firstV; c < obj[currPick + 1].numV; c++)
	{
		Xold = v[c].x;
		Yold = v[c].y;
		Xnew = cos(a) * Xold - sin(a) * Yold;
		Ynew = sin(a) * Xold + cos(a) * Yold;
		v[c].x = Xnew;
		v[c].y = Ynew;		
	}
	
	pickAngle += a2;
}

GLvoid KillFont( GLvoid )
{
    glDeleteLists( base, 96 );

    return;
}

/* function to release/destroy our resources and restoring the old desktop */
void Quit( int returnCode )
{
	 /* Recover some memory */
    XFreeFont( dpy, fontInfo );
 	XFreeFont( dpy, fontInfo2 );
    /* close the display now that we're done with it */
    XCloseDisplay( dpy );
	
	KillFont( );
    /* clean up the window */
    SDL_Quit( );

    /* and exit appropriately */
    exit( returnCode );
}

/* function to build our font list */
GLvoid buildFont(GLvoid)
{
            /* Our current X display */
 
    /* Sotrage for 96 characters */
    base = glGenLists( 96 );

    /* Get our current display long enough to get the fonts */
    dpy = XOpenDisplay( NULL );

    /* Get the font information */
    fontInfo = XLoadQueryFont( dpy, "-adobe-helvetica-medium-r-normal--18-*-*-*-p-*-iso8859-1" );
	fontInfo2 = XLoadQueryFont( dpy, "-adobe-helvetica-medium-r-normal--32-*-*-*-p-*-iso8859-1" );
	
    /* If the above font didn't exist try one that should */
    if ( fontInfo == NULL )
	{
	    fontInfo = XLoadQueryFont( dpy, "fixed" );
	    /* If that font doesn't exist, something is wrong */
	    if ( fontInfo == NULL )
		{
		    fprintf( stderr, "no X font available?\n" );
		    Quit( 1 );
		}
	}

    /* generate the list */
    



    return;
}

/* Print our GL text to the screen */
GLvoid glPrint( const char *fmt, ... )
{
    char text[256]; /* Holds our string */
    va_list ap;     /* Pointer to our list of elements */

    /* If there's no text, do nothing */
    if ( fmt == NULL )
	return;

    /* Parses The String For Variables */
    va_start( ap, fmt );
      /* Converts Symbols To Actual Numbers */
      vsprintf( text, fmt, ap );
    va_end( ap );

    /* Pushes the Display List Bits */
    glPushAttrib( GL_LIST_BIT );

    /* Sets base character to 32 */
    glListBase( base - 32 );

    /* Draws the text */
    glCallLists( strlen( text ), GL_UNSIGNED_BYTE, text );

	SDL_Delay(1);
	
    /* Pops the Display List Bits */
    glPopAttrib( );
}


/*LOADMODEL-------------------------
This function loads 3d models of 
.obj format inlcuding normal data
----------------------------------*/

void loadModel(char * nam, char * filename, float scale, float xx, float yy, float zz, float pinlength)
{
	FILE *fp;
	fp = fopen(filename, "r");

	if (fp == NULL)
	{
		printf("Error: model %s not found\n", filename);
		Quit(0);
	}
	
	char test[100];
	int maxChars = 100;

	int lastV = totalV;
	int lastVn = totalVn;
	
	obj[totalObj].firstV = lastV;
	
	obj[totalObj].name = nam;
	
	while(!feof(fp))
	{
		int cc = 0;
		fgets(test, maxChars, fp);

			
		char args[10][50];
		
		char * tok = strtok(test, " ");
		while (tok != NULL) 
		{
			strcpy(args[cc], tok);
			cc++;
			tok = strtok(NULL," ");
		}
		
		if (strcmp(args[0], "v") == 0)
		{
			v[totalV].x = atof(args[1]);
			v[totalV].y = atof(args[2]);
			v[totalV].z = atof(args[3]);
			
			if (v[totalV].y > 0.0f) v[totalV].y -= pinlength;	
			
			
			totalV++;
			
			
		}
		else if (strcmp(args[0], "vn") == 0)
		{
			vn[totalVn].x = atof(args[1]);
			vn[totalVn].y = atof(args[2]);
			vn[totalVn].z = atof(args[3]);
			
			totalVn++;
			
		}
		else if (strcmp(args[0], "f") == 0)
		{

			face[maxQuads].type = 3;
			
			char * tok2 = strtok(args[1], "//");
			face[maxQuads].v1 = atoi(tok2) + lastV - 1;
			tok2 = strtok(NULL,"//");
			if (tok2 != NULL) face[maxQuads].fn = atoi(tok2) + lastVn - 1;
			
			tok2 = strtok(args[2], "//");
			face[maxQuads].v2 = atoi(tok2) + lastV - 1;
			tok2 = strtok(args[3], "//");
			face[maxQuads].v3 = atoi(tok2) + lastV - 1;
			
			
			if (args[4] != NULL && atoi(args[4]) > 0)			
			{
				tok2 = strtok(args[4], "//");
				face[maxQuads].v4 = atoi(tok2) + lastV - 1;
				face[maxQuads].type = 4;
				
			}
			else
			{
					
			}
			
			obj[totalObj].f[obj[totalObj].maxF] = maxQuads;
			obj[totalObj].maxF++;
			
			maxQuads++;
			
		}
		
	}
	
	obj[totalObj].numV = totalV;
	
	obj[totalObj].transX = xx;
	obj[totalObj].transY = yy;
	obj[totalObj].transZ = zz;
	obj[totalObj].scale = scale;
	
	totalObj++;

	fclose(fp);
	return;
}



/* function to reset our viewport after a window resize */
int resizeWindow( int width, int height )
{
    /* Height / width ration */
    GLfloat ratio;

    /* Protect against a divide by zero */
    if ( height == 0 )
	height = 1;

    ratio = ( GLfloat )width / ( GLfloat )height;

    /* Setup our viewport. */
    glViewport( 0, 0, ( GLsizei )width, ( GLsizei )height );

    /* change to the projection matrix and set our viewing volume. */
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );

    /* Set our perspective */
    gluPerspective( 45.0f, ratio, 0.1f, 400.0f );

    /* Make sure we're chaning the model view and not the projection */
    glMatrixMode( GL_MODELVIEW );

    /* Reset The View */
    glLoadIdentity( );

    return( TRUE );
}

void handleMouse(int xrel, int yrel, int x, int y, int state)
{
	if (mode == 2)
		return;
	
	
	/*Mouse Higher*/
	
	int c = 0;
	int d = 0;
	

	
	for(d = 0; d < 10000; d++)
	{
		if (obj[currPick + 1].firstV + d < obj[currPick + 1].numV)
		{
			
			if ((obj[currPick].transX + v[obj[currPick + 1].firstV + d].x > 29.2 - pinWidth)
			&& (obj[currPick].transX + v[obj[currPick + 1].firstV + d].x < 29.2 + pinWidth))
			{
				if ((obj[currPick].transY + v[obj[currPick + 1].firstV + d].y + pickOffset - 0.2f < 1.0f)
					&& (obj[currPick].transY + v[obj[currPick + 1].firstV + d].y  + pickOffset + 0.03f >= -0.2f))
				{
					if (yrel <= 0 && (!clicked)) 
					{
						yrel = 1;
						xrel = 0;
					}
					if (yrel >= 0 && (clicked))
					{
						yrel = -1;	
						xrel = 0;

					}
				}
			}
		}
		else
		{
			break;
		}
	}
	
	if (clicked)
	{
		rotate( ((float)yrel) / 20.0f  );
	}
	
	
	for (c = 0; c < startTop; c++)
	{

		pinPushing[c] = 0;	
		int crappy = 0;
		
		for(d = 0; d < 10000; d++)
		{
			if (obj[currPick + 1].firstV + d < obj[currPick + 1].numV)
			{
			
			if ((obj[currPick].transX + v[obj[currPick + 1].firstV + d].x > obj[c].transX - pinWidth)
			&& (obj[currPick].transX + v[obj[currPick + 1].firstV + d].x < obj[c].transX + pinWidth))
			{
								
				
				if ((obj[currPick].transY + v[obj[currPick + 1].firstV + d].y + pickOffset - 0.2f < pinPush[c])
					&& (obj[currPick].transY + v[obj[currPick + 1].firstV + d].y  + pickOffset + 0.03f >= pinPush[c]))
				{				
					pinPushing[c] = 1;
					
					if (pinPushingContour[c] != -1)
					{
					if((pinPushingContour[c] != obj[currPick + 1].firstV + d)|| (TRUE))
					{
						if (v[pinPushingContour[c]].y + 0.4f < v[obj[currPick + 1].firstV + d].y  + pickOffset)
						{
								
								pinPush[c] += (v[obj[currPick + 1].firstV + d].y + pickOffset) - (v[pinPushingContour[c]].y + pickOffset);

						}	
						else
						{	if (TRUE)
								pinPush[c] += (v[obj[currPick + 1].firstV + d].y + pickOffset) - (v[pinPushingContour[c]].y + pickOffset);
						}

												
					}
					}
					
					pinPushingContour[c] = obj[currPick + 1].firstV + d;
					crappy = 1;
					
					break;	
				}
				else
				{
					if ((obj[currPick].transY + v[obj[currPick + 1].firstV + d].y + pickOffset < -0.4f))
					{
						
					}
					else
					{
						if (xrel > 0) xrel = 0;
					}						
				}					
				
			}
		}
		else
		{	
			break;	
		}
	}
	
	if (crappy == 0) pinPushingContour[c] = -1;
	
	}
	
	
	if (xrel > 0)
		obj[currPick].transX += 0.005f * (float)xrel;
	if (xrel < 0)
		obj[currPick].transX += 0.005f * (float)xrel;
	
	if (!clicked)
	{
	if (yrel > 0)
	{
		obj[currPick].transY -= 0.005f * (float)yrel;
		
	}
	
	
	if (yrel < 0)
	{
		obj[currPick].transY -= 0.005f * (float)yrel;

	}
	}
	
	if (clicked) yrel *= -1;
	
	for(c = 0; c < startTop; c++)
	{
		
		if ((yrel > 0) && pinOverset[c] == 0)
		{	
				if (pinPush[c] > -0.5f)
				{
					
						pinPush[c] -= ((float)yrel * 0.005f);
				}
				else
					pinPush[c] = -0.5f;
			
		}
		else if ( (yrel < 0) && pinPushing[c] == 1 )
		{
			if(obj[currPick].transY + 0.4f + v[pinPushingContour[c]].y  > pinPush[c])
			{
				
					pinPush[c] -= ((float)yrel * 0.005f);
			}
		}

		
	}
}


/* function to handle key press events */
void handleKeyPress( SDL_keysym *keysym, int details )
{

	
	int c = 0;
	
    switch ( keysym->sym )
	{
	case SDLK_ESCAPE:
	    /* ESC key was pressed */
	    mode = 2;
		currMenu = 0;
		menuSel = 0;
	    break;
	case SDLK_F1:
	    /* F1 key was pressed
	     * this toggles fullscreen mode
	     */
	    SDL_WM_ToggleFullScreen( surface );
	    break;
	case SDLK_c:
		loadLock(currLock);	
	
		obj[currPick].transX = 28;
		obj[currPick].transY = -2.0;
		obj[currPick].transZ = 0;
	
		timeOn = 1;
		timeStart = t;
	
		tension = 0.0f;
	
		break;
	case SDLK_UP:
		
		
	
		/*SCROLL UP THRU MENU*/
		
		if (keyUp == 0 && mode == 2)
		{
			if (currMenu == 0 && menuSel > 0)
			{
				menuSel--;
			}
			else if (currMenu == 1 && menuSel > 0)
			{
				menuSel--;
			}
			else if (currMenu == 2 && menuSel > 0)
			{
				menuSel--;
			}
		}
		keyUp = details;
		
		break;
	case SDLK_x:
		if (keyX == 0 && mode == 0)
		{
			for(c = 0; c < startTop; c++)
			{
				pinOverset[c] = 0;
				pinSet[c] = 0;
				pinNotchSet[c] = 0;
				falseSet[c] = 0;
			}
			plugRot = 180.0f;
			int e = 0;
			for(e = 0; e < startTop; e++)
			{
				pinOrder[e] = pinOrderPerm[e];
				
			}
			

			
			tension = 0.0f;
		}	
		
		keyX = details;
		break;
	case SDLK_DOWN:
		if (keyDown == 0 && mode == 0)
		{
			
		}			
		else if (keyDown == 0 && mode == 2)
		{
			/*SCROLL DOWN THRU MENU*/
			
			if (currMenu == 0 && menuSel < 4)
			{
				menuSel++;
			}
			else if (currMenu == 1 && menuSel < 3)
			{
				menuSel++;
			}
			else if (currMenu == 2 && menuSel < 1 + lock[currLock].numPins * 2 + 5)
			{
				menuSel++;
			}
			

		}
		
	
		keyDown = details;
		break;
	case SDLK_RETURN:
		if (mode == 2)
		{
			if (currMenu == 0)
			{				
				if (menuSel == 0)
				{				
					currMenu = 1;	
				}
				else if (menuSel == 1)
				{				
					currMenu = 2;
					for (c = 0; c < 100; c++)
					{
						if (lock[c].name == NULL)
						{
							break;
						}
					}				

					lock[c].name = "Custom";
					lock[c].tolerance = 0.05f;
					lock[c].numPins = 5;
					
					int craptastic;
					
					for(craptastic = 0; craptastic < lock[c].numPins; craptastic++)
						lock[c].pinLen[craptastic] = 0.45f;	
					
					currLock = c;
					menuSel = 0;	
					
					
				}
				else if (menuSel == 3)
				{
					currMenu = 10;
				}
				else if (menuSel == 4)
				{
					Quit(0);
					return;
				}
			}
			else if(currMenu == 1)
			{
				if (menuSel == 2)
				{
					loadLock(currLock);
					mode = 0;
					
					timeOn = 1;
					timeStart = t;
					
					obj[currPick].transX = 28;
					obj[currPick].transY = -2.0;
					obj[currPick].transZ = 0;
			
					tension = 0.0f;
				}
			}
			else if (currMenu == 2 && details == 1)
			{
				if (menuSel == 2 + lock[currLock].numPins * 2)
				{
					currMenu = 0;
					menuSel = 0;
				}
				else if (menuSel == 2 + lock[currLock].numPins * 2 + 1)
				{
					for (c = 0; c < lock[currLock].numPins; c++)
					{
						lock[currLock].pinLen[c] = (rand() % 10 + 1) * 5.0f / 100.0f;
					
						
					}
				}
				else if (menuSel == 2 + lock[currLock].numPins * 2 + 2)
				{
					for (c = 0; c < lock[currLock].numPins; c++)
					{
						lock[currLock].topin[c] = (int)(rand() % 4);

					}
				}
				else if (menuSel == 2 + lock[currLock].numPins * 2 + 3)
				{
					for (c = 0; c < lock[currLock].numPins; c++)
					{
						lock[currLock].pinLen[c] = (rand() % 10 + 1) * 5.0f / 100.0f;
						lock[currLock].topin[c] = (int)(rand() % 4);

					}
				}
				else if (menuSel == 2 + lock[currLock].numPins * 2 + 4)
				{
					lock[currLock].tolerance = rand() % 100 / 1000.0f;
					
					for (c = 0; c < lock[currLock].numPins; c++)
					{
						lock[currLock].pinLen[c] = (rand() % 10 + 1) * 5.0f / 100.0f;
						lock[currLock].topin[c] = (int)(rand() % 4);

					}
				}
			}
		}
		
		break;
	case SDLK_LEFT:
		if (mode == 0)
			keyLeft = details;
		else if (mode == 1 && keyLeft == 0)
		{
			if (currPick - 2 >= startSprings + startTop)
			{
				
				currPick -= 2;
				
			}
		}
		else if (mode == 2 && keyLeft == 0) 
		{
			if (currMenu == 1 && currLock > 0 && menuSel == 0)
			{
				currLock--;
			}
			else if (currMenu == 1 && menuSel == 1)
			{
				if (strcmp(visibility, "Plug") == 0)
					visibility = "Full";
				else if (strcmp(visibility, "Realistic") == 0)
					visibility = "Plug";
			}
			else if (currMenu == 2 && menuSel == 0)
			{
				if (lock[currLock].tolerance > 0.005f)
					lock[currLock].tolerance -= 0.005f;
			}
			else if (currMenu == 2 && menuSel == 1)
			{
				if (lock[currLock].numPins > 1)
					lock[currLock].numPins--;
			}
			else if (currMenu == 2 && menuSel > 1 && (menuSel - 2) < lock[currLock].numPins)
			{
				if (lock[currLock].pinLen[menuSel - 2] < 0.45)
					lock[currLock].pinLen[menuSel - 2] += 0.05f;
			}
			else if (currMenu == 2 && (menuSel -2) >= lock[currLock].numPins && (menuSel-2) < lock[currLock].numPins * 2)
			{
				if (lock[currLock].topin[menuSel - lock[currLock].numPins - 2] > 0)
					lock[currLock].topin[menuSel - lock[currLock].numPins - 2]--;
			}
		}
		keyLeft = details;
		break;
	case SDLK_RIGHT:
		
		if (mode == 0)
			keyRight = details;
		else if (mode == 1 && keyRight == 0)
		{
			if (currPick + 2 < totalObj)
			{
				currPick += 2;
				
			}
		}
		else if (mode == 2 && keyRight == 0)
		{
			if (currMenu == 1 && lock[currLock + 1].name != NULL && menuSel == 0)
				currLock++;
			else if (currMenu == 1 && menuSel == 1)
			{
				if (strcmp(visibility, "Full") == 0)
					visibility = "Plug";
				else if (strcmp(visibility, "Plug") == 0)
					visibility = "Realistic";
			}
			else if (currMenu == 2 && menuSel == 0)
			{
				if (lock[currLock].tolerance < 0.1f)
					lock[currLock].tolerance += 0.005f;
			}
			else if (currMenu == 2 && menuSel == 1)
			{
				if (lock[currLock].numPins < 7)
					lock[currLock].numPins++;
			}
			else if (currMenu == 2 && menuSel > 1 && (menuSel - 2) < lock[currLock].numPins)
			{
				if (lock[currLock].pinLen[menuSel - 2] > 0.10f)
					lock[currLock].pinLen[menuSel - 2] -= 0.05f;
			}
			else if (currMenu == 2 && (menuSel -2) >= lock[currLock].numPins && (menuSel-2) < lock[currLock].numPins * 2)
			{
				if (lock[currLock].topin[menuSel - lock[currLock].numPins - 2] < 3)
					lock[currLock].topin[menuSel - lock[currLock].numPins - 2]++;
			}
		}
		
		keyRight = details;
		break;
	case SDLK_SPACE:
		keySpace = details;
		break;
	case SDLK_z:
		if (details)
		{
			printf("%f\n", pickAngle);
			
			rotate(-pickAngle);
			
			if (mode == 0)
				mode = 1;
			else
				mode = 0;
		}			
	
		break;
	case SDLK_q:
		rotate(1.0f);	
	
		break;
	default:
	    break;
	}

	
    return;
}

/* general OpenGL initialization function */
int initGL( GLvoid )
{

    /* Enable smooth shading */
    glShadeModel( GL_SMOOTH );
	glDisable( GL_DITHER);
	
	
    /* Set the background black */
    glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );

    /* Depth buffer setup */
    glClearDepth( 1.0f );

    /* Enables Depth Testing */
    glEnable( GL_DEPTH_TEST );
		
    /* The Type Of Depth Test To Do */
    glDepthFunc( GL_LEQUAL );

    /* Really Nice Perspective Calculations */
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

	glEnable(GL_LIGHTING);
	
	glEnable(GL_LIGHT1);
	glDisable(GL_NORMALIZE);

	glCullFace(GL_FRONT);
	/*glEnable(GL_CULL_FACE);*/
	
	buildFont( );
	
    return( TRUE );
}

/* Here goes our drawing code */
int drawGLScene( GLvoid )
{
    /* These are to calculate our fps */
    static GLint T0     = 0;
    static GLint Frames = 0;
    static GLint M0     = 0;

	GLfloat xtrans = -cameraX;
	GLfloat ztrans = -cameraZ;
	GLfloat ytrans = -cameraY;
	GLfloat scenerot = 360.0f - cameraRotY;

    /* Clear The Screen And The Depth Buffer */
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    /* Move Left 1.5 Units And Into The Screen 6.0 */
	if (mode != 2)
	{
    glLoadIdentity();
	

    glRotatef(scenerot, 0.0f, 1.0f, 0.0f);	
    glTranslatef(xtrans, ytrans, ztrans);


	
	GLfloat LightAmbient[]= { 0.5f, 0.5f, 0.5f, 5.0f };
	GLfloat LightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat LightPosition[]= { 20.0f, 3.0f, 40.0f, 1.0f };
	GLfloat LightPosition2[]= { 20.0f, 0.0f, 40.0f, 1.0f };
	
	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);
	glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);
	
	glLightfv(GL_LIGHT2, GL_AMBIENT, LightAmbient);
	glLightfv(GL_LIGHT2, GL_DIFFUSE, LightDiffuse);
	glLightfv(GL_LIGHT2, GL_POSITION,LightPosition2);
	}

    /*glRotatef(cameraZ, 0.0f + cameraX, 0.0f + cameraY, -6.0f + cameraZ );*/

int c = 0;
int d = 0;
int e = 0;

	
if (mode != 2)
{
	for(e = 0; e < totalObj; e++)
	{
		
		
		if (e >= startSprings + startTop && currPick != e && mode == 0)
			continue;
		if (obj[e].invisible)
			continue;			

		

		glPushMatrix();

		
		glColor3f(0.5f, 0.5f, 0.5f);
	
		if (e >= startSprings + startTop && mode == 1)
		{
			glLoadIdentity();
			

			glTranslatef(e * 0.012f - (startSprings + startTop) * 0.012f - 0.1f, 0.021f, -0.2f);
			glRotatef(90.0f, 0.0f, 0.0f, 1.0f);
			
			if (currPick == e)
				glScalef(0.010f, 0.010f, 0.010f);
			else
				glScalef(0.008f, 0.008f, 0.008f);
			
		}	

		
		if ((e < startSprings + startTop || mode == 0) && e != currPick)
		{
			glTranslatef(obj[e].transX, obj[e].transY, obj[e].transZ);
			glScalef(1.0f, obj[e].scale, 1.0f);
		}

		
		
		if (e < startTop )
		{
			
			
			glTranslatef(0.0f, -1.0f, 0.0f);
		 	glRotatef(plugRot, 1.0f, 0.0f, 0.0f);
			glTranslatef(0.0f, -1.0f, 0.0f);
		 	glRotatef(180.0f, 1.0f, 0.0f, 0.0f);
			glTranslatef(0.0f, pinPush[e], 0.0f);
		}
		
		if (e == startTop + startTop || e == startTop + startTop + 1 || e == startTop + startTop + 2 || e == currPick)
		{
			
			
			if (e != currPick)
			{
				glTranslatef(0.0f, -1.0f, 0.0f);
				glRotatef(plugRot - 180.0f, 1.0f, 0.0f, 0.0f); 
			}
			


			if (e == currPick && mode == 0)
			{
				
				glTranslatef(0.0f, -1.0f, 0.0f);
				glRotatef(plugRot + 180.0f, 1.0f, 0.0f, 0.0f);
				glTranslatef(obj[e].transX, obj[e].transY, obj[e].transZ);
			}
			
			
			if (e == startTop + startTop)
				glTranslatef(0.0f, -0.75f, 0.0f);
			else if (e == startTop + startTop + 1)
			{
				glTranslatef(0.0f, -0.5f - tension / 2, 4.5f);
			
			}
		}
		
		glCullFace(GL_BACK);
		
		if (e >= startSprings && e < startSprings + startTop) glCullFace(GL_FRONT);
				

	for(d = 0; d < obj[e].maxF; d++)
	{
		
		if (face[obj[e].f[d]].type == 4)
			glBegin( GL_QUADS );
		else
			glBegin( GL_TRIANGLES);
		
		glNormal3f(vn[face[obj[e].f[d]].fn].x, vn[face[obj[e].f[d]].fn].y, vn[face[obj[e].f[d]].fn].z);		
		glVertex3f(v[face[obj[e].f[d]].v1].x, v[face[obj[e].f[d]].v1].y, v[face[obj[e].f[d]].v1].z);
		glVertex3f(v[face[obj[e].f[d]].v2].x, v[face[obj[e].f[d]].v2].y, v[face[obj[e].f[d]].v2].z);
		glVertex3f(v[face[obj[e].f[d]].v3].x, v[face[obj[e].f[d]].v3].y, v[face[obj[e].f[d]].v3].z);
		
		if (face[d].type == 4)
		{
			glVertex3f(v[face[obj[e].f[d]].v4].x, v[face[obj[e].f[d]].v4].y, v[face[obj[e].f[d]].v4].z);
		}
		
		glEnd( );

	}
	
		glPopMatrix();
				
		
}  

	glDisable(GL_LIGHTING);
	

	glLoadIdentity();	
	glTranslatef( 0.30f, 0.35f, -1.0f );
    glColor3f( 1.0, 1.0f, 1.0f);
	glRasterPos2f( 0, 0);
    glPrint( "Time: %7.2f", ((float)timeEnd) / 1000.0f );
		
	glTranslatef( 0.005f, -0.03f, 0.0f );		
	glRasterPos2f( 0, 0);
    glPrint( "Pick: %7s", obj[currPick].name );

	glEnable(GL_LIGHTING);

}
else
{
	/*MENU CODE HERE*/
	
	glLoadIdentity();
	
	glDisable(GL_LIGHTING);
	
	
	
	if (currMenu == 0)
	{
		glXUseXFont( fontInfo->fid, 32, 96, base );
		
		glColor3f( 1.0, 0.5f, 0.0f);
		
		glTranslatef( -0.3f, 0.3f, -1.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("NDE Presents");
		
		
		glXUseXFont( fontInfo2->fid, 32, 96, base );
		glColor3f( 1.0, 1.0f, 1.0f);
		glTranslatef( 0.0f, -0.05f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("LockGame 3D");
		
		glXUseXFont( fontInfo->fid, 32, 96, base );
		/*MAIN MENU*/

		if (menuSel != 0)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		

		
		glTranslatef( 0.0f, -0.1f, -0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Play");
		
		if (menuSel != 1)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		glTranslatef( 0.0f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("Custom");
		
		

		if (menuSel != 2)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		glTranslatef( 0.0f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("Settings");
		
		if (menuSel != 3)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		glTranslatef( 0.0f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		
		
		glPrint("Credits");
		
		if (menuSel != 4)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		glTranslatef( 0.00f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("Quit");
		
	} 
	else if (currMenu == 1)
	{
		/*PLAY MENU*/
		
		glXUseXFont( fontInfo2->fid, 32, 96, base );
		glColor3f( 0.5f, 1.0f, 0.5f);
		glTranslatef( -0.5f, 0.3f, -1.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Play");
		
		glXUseXFont( fontInfo->fid, 32, 96, base );
		glColor3f( 2.0, 2.0f, 2.0f);
		glTranslatef( 0.0f, -0.1f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Lock:");
		
		if (menuSel != 0)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( 0.7f, 0.0f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("%i: %s", currLock, lock[currLock].name);
		
		glColor3f( 2.0, 2.0f, 2.0f);
		glTranslatef( -0.7f, -0.03f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Visibility:");
		
		if (menuSel != 1)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( 0.7f, 0.0f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint(visibility);
		
		if (menuSel != 2)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( -0.35f, -0.06f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("<Play>");
		
		glTranslatef( -0.35f, -0.06f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Fasest Time for Visibility:");
		
		float timey = lock[currLock].fastTimeFull;
		
		if (strcmp(visibility, "Plug") == 0) timey = lock[currLock].fastTimePlug;
		if (strcmp(visibility, "Realistic") == 0) timey = lock[currLock].fastTimeRealistic;
				
		glTranslatef( 0.7f, 0.0f, 0.0f );
    	glRasterPos2f( 0, 0);
        if (timey > 0.0f)
        	glPrint( "%7.2f seconds", timey / 1000.0f );
       	else
       	    glPrint("Unopened");
		
	}
	else if (currMenu == 2)
	{
		/*CUSTOM MENU*/
		
		glXUseXFont( fontInfo2->fid, 32, 96, base );
		glColor3f( 0.5f, 1.0f, 0.5f);
		glTranslatef( -0.5f, 0.3f, -1.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Custom");
		
		glXUseXFont( fontInfo->fid, 32, 96, base );
		glColor3f( 2.0, 2.0f, 2.0f);
		glTranslatef( 0.0f, -0.1f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Lock Name:");
		glTranslatef( 0.7f, 0.0f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint(lock[currLock].name);
		
		glColor3f( 2.0, 2.0f, 2.0f);
		glTranslatef( -0.7f, -0.03f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Tolerance:");
		
		if (menuSel != 0)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( 0.7f, 0.0f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("%i%%", (int)(lock[currLock].tolerance * 1000.0f));
		
		glColor3f( 2.0, 2.0f, 2.0f);
		glTranslatef( -0.7f, -0.03f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("# of Pins:");
		
		if (menuSel != 1)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( 0.7f, 0.0f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("%i", lock[currLock].numPins);
		
		glTranslatef( -0.4f, -0.03f, 0.0f);
		
		for(c = 0; c < lock[currLock].numPins; c++)
		{
		
		glColor3f( 2.0, 2.0f, 2.0f);
		glTranslatef( -0.3f, -0.03f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Pin %i Length:", c + 1);
		
		if (menuSel != 2 + c)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( 0.3f, 0.0f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("%i", (int)(10 - (lock[currLock].pinLen[c] / 5.0f * 100.0f) + 1));
		
		}
		
		glTranslatef( 0.5f, lock[currLock].numPins * 0.03f, 0.0f);
		
		for(c = 0; c < lock[currLock].numPins; c++)
		{
		
		glColor3f( 2.0, 2.0f, 2.0f);
		glTranslatef( -0.3f, -0.03f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Driver %i Type:", c + 1);
		
		if (menuSel != 2 + c + lock[currLock].numPins)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( 0.3f, 0.0f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("%s", pin[lock[currLock].topin[c]].name);
		
		}
		
		if (menuSel != 2 + lock[currLock].numPins * 2)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( -0.8f, -0.06f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("<Save Current>");
		
		if (menuSel != 2 + lock[currLock].numPins * 2 + 1)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( 0.0f, -0.03f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("<Random Pin Lengths>");
		
		if (menuSel != 2 + lock[currLock].numPins * 2 + 2)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( 0.0f, -0.03f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("<Random Top Pins>");
		
		if (menuSel != 2 + lock[currLock].numPins * 2 + 3)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( 0.0f, -0.03f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("<Random Pins>");
		
		if (menuSel != 2 + lock[currLock].numPins * 2 + 4)
			glColor3f( 2.0, 2.0f, 2.0f);
		else
			glColor3f( 2.0, 2.0f, 0.0f);
		
		glTranslatef( 0.0f, -0.03f, 0.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("<Random Lock>");
		
	}
	else if (currMenu == 10)
	{
		/*CREDITS*/
		
		glXUseXFont( fontInfo2->fid, 32, 96, base );
		glColor3f( 0.5f, 1.0f, 0.5f);
		glTranslatef( -0.5f, 0.3f, -1.0f );
    	glRasterPos2f( 0, 0);	
		glPrint("Credits");
		
		glXUseXFont( fontInfo->fid, 32, 96, base );
				
		glColor3f( 2.0, 2.0f, 2.0f);
		glTranslatef( 0.0f, -0.05f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("Programming/Design/Modelling: JKtheCJer");
		glTranslatef( 0.0f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("Distribution/Publishing: Schuyler + NDEmag");
		glTranslatef( 0.0f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("Inspiration/Ideas: The Locksport Community esp. lp101");
		
		glTranslatef( 0.0f, -0.06f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("Notes:");
		glTranslatef( 0.00f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("   This has been my first experience with both SDL and OpenGL.");
		glTranslatef( 0.00f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("   It is also my first major project developed under Linux.");
		glTranslatef( 0.00f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		
		glPrint("   I am planning a complete rewrite for the next version of this app.");
		glTranslatef( 0.00f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("   Any praise, criticism, suggestions, etc... is welcome (jkthecjer@gmail.com");
		glTranslatef( 0.00f, -0.03f, 0.0f );
		glRasterPos2f(0, 0);
		glPrint("   This is only the beginning...");
		
	}

	
}

    /* Draw it to the screen */

    SDL_GL_SwapBuffers( );

    /* Gather our frames per second */
    Frames++;
    {
	t = SDL_GetTicks();

	if (timeOn) timeEnd = t - timeStart;
	
	
	if (t - T0 >= 1000)
	{
	    GLfloat seconds = (t - T0) / 1000.0;
	    GLfloat fps = Frames / seconds;
	    printf("%d frames in %g seconds = %g FPS\n", Frames, seconds, fps);
	    
		T0 = t;
	    Frames = 0;
	}

	if (mode == 2)
	{
		return (TRUE);
	}
	
	if (t - M0 >= 5) 
	{
		if (keyUp)
		{
			cameraX -= (float)sin(cameraRotY * piover180) * 0.5f;
			cameraZ -= (float)cos(cameraRotY * piover180) * 0.5f;
			cameraY += (float)sin(cameraRotX * piover180) * 0.5f;

				

		}		

		if (keyLeft && mode == 0) cameraRotY += 1.0f;
		if (keyRight && mode == 0) cameraRotY -= 1.0f;
		

		if (keySpace)
		{
			if (tension < maxTension) tension += tensionRate;		
		}
		
		if (!keySpace)
		{
			if (tension >= 0.0f)
				tension -= tensionLoss;
			else
				tension = 0.0f;
		}
			
		/*MORE MOUSE STUFF-------------------------------------------*/
		
		for(c = 0; c < startTop; c++)
		{
			if (pinPushing[c] == 0 && pinOverset[c] == 0)
			{
						
				if (pinPush[c] <= -0.5f)
					pinPush[c] = -0.5f;
				else
				{
					if (obj[currPick].transY + v[pinPushingContour[c]].y + 0.04f > pinPush[c])	
					{						

					}
					else
					{
						pinPush[c] = obj[currPick].transY + v[pinPushingContour[c]].y + 0.04f;
					}
				}
			}
			else
			{
				
			}
		}
		
		
		for(c = 0; c < obj[startTop + startTop + 1].maxF; c++)
		{
			if ((v[face[obj[startTop + startTop + 1].f[c]].v1].y >= -0.1f) && (v[face[obj[startTop + startTop + 1].f[c]].v2].y >= -0.1f) && 
				(v[face[obj[startTop + startTop + 1].f[c]].v3].y >= -0.1f))
			{
				v[face[obj[startTop + startTop + 1].f[c]].v1].y = tension;
				v[face[obj[startTop + startTop + 1].f[c]].v2].y = tension;
				v[face[obj[startTop + startTop + 1].f[c]].v3].y = tension;
				if (face[obj[startTop + startTop + 1].f[c]].type == 4) 
					v[face[obj[startTop + startTop + 1].f[c]].v4].y = tension;
			}	
		}
		
		
		
		int lockOpen = 1;
		
		for(c = 0; c < startTop; c++)
			if (pinSet[c] == 0)
			{
				lockOpen = 0;
			}
		
		if (lockOpen == 1)
		{
			timeOn = 0;
			
			if (strcmp(visibility, "Full") == 0)
			{
				if (lock[currLock].fastTimeFull > timeEnd || lock[currLock].fastTimeFull == 0.0f)
			       lock[currLock].fastTimeFull = timeEnd;
            }
            else if (strcmp(visibility, "Plug") == 0)
            {
				if (lock[currLock].fastTimePlug > timeEnd || lock[currLock].fastTimePlug == 0.0f)
			       lock[currLock].fastTimePlug = timeEnd;                 
            }
            else
            {
 				if (lock[currLock].fastTimeRealistic > timeEnd || lock[currLock].fastTimeRealistic == 0.0f)
			       lock[currLock].fastTimeRealistic = timeEnd;               
            }
			
			
			if (keySpace) plugRot += 4.0f;
				
			if (plugRot >= 540.0f)
			{
				for (c = 0; c < startTop; c++)
					pinSet[c] = 0;
			} 
			tension = 0.0f;	
		}			
		
		if((tension <= minTension) && (lockOpen == 0))
		{
			int e = 0;
			for(e = 0; e < startTop; e++)
			{
				pinSet[e] = 0;
				pinOverset[e] = 0;
				pinOrder[e] = pinOrderPerm[e];
				pinNotchSet[e] = 0;
				falseSet[e] = 0;

			}
			plugRot = 180.0f;

		}
		
		for(c = 0; c < startTop; c++)
		{
			obj[c + startSprings].scale = (obj[c + startTop].transY * 1.5f - 1.4f);
			obj[c + startSprings].transY = 1.2f + (obj[c + startSprings].scale * 0.3f);	
		}
		
		/*BASIC PHYSICS-------------------------------------------*/
		

		for(c = startTop; c < startTop + startTop; c++)
		{
			if (obj[c].gravity == TRUE)
			{
				
				d = c - startTop;
				
				if (pinPushingContour[d] == -1 && pinOverset[d] == 0) 
					if (falseSet[d] == 0)
						pinPush[d] -= GRAVITY / 2;
				
				
				if (pinPushing[d] == 0 && pinOverset[d] == 0)
				{

				}
				
				if ((pinPushing[d]) && (pinOrder[d] <= 0))
				{
					if(tension > minTension)
					{
						
						int tempa = (int)((tension - minTension) * 10.0f);
						
						if (tempa == 0)
							tempa = 1;
						
						tempa = (rand() % tempa);
						
						
						
						if (tempa != 0)
						{
							float pinRes = pinResistance;
							
							if (pinPush[d] > shearline + pinLen[d])
							{
								if (pinSet[d]) 
								{
									pinRes += pinSetResistance;
								}
								if (pinOverset[d]) 
								{
									if (pinPush[d] > shearline + pinLen[d])								
										pinRes += pinOversetResistance;
									else
										pinRes = 0;
								
								}							
							}
							else
							{
								if (pinSet[d])
									pinRes = 0.0f;
								
								
							}
							
							if (pinPush[d] > -0.5f)
							{
							
							if (pinOverset[d]) pinRes *= 2;
								
							int dd = 0;
							
							for(dd = 0; dd < startTop; dd++)
								if (!pinOverset[dd] && !falseSet[dd])
									pinPush[dd] -= pinRes;

							if (!(pinOverset[d] || falseSet[d]))
							{
								if (!clicked)
									obj[currPick].transY -= pinRes;
								else
									rotate(pinRes * -10);
							}
							}		
						}	
					}						
				}
				
				if (pinOrder[d] == 0 && tension > minTension)
				if (pinType[d] != 0 && pinSet[d] == 0 && pinOverset[d] == 0)
								{
								
								/*SECURITY PIN CODE*/
								int ee = 0;
								int crackhead = 0;
								
								for(ee = 0; ee < 10; ee++)
								{
									if ((pinPush[d] < shearline + pinLen[d] - pin[pinType[d]].notchEnd[ee])
										&& (pinPush[d] > shearline + pinLen[d] - pin[pinType[d]].notchStart[ee]))
									{
										if (pinNotchSet[d] == 0)
										{
											pinNotchSet[d] = 1;
											
											if (falseSet[d] == 0)
											   	plugRot += setRot;
											
											falseSet[d] = 1;
											/*COMING INTO NOTCH*/
										}
										crackhead = 1;										
									}
								}
								
								if (crackhead == 0)
								{
									if (pinNotchSet[d] == 1 && obj[c].transY <= obj[d].boundhigh + obj[d].transY + pinPush[d] - obj[c].boundlow + 0.001f)
									{
										/*COMING OUT OF NOTCH*/
										int testme = rand() % notchCrap;
										
										if (testme == 0)
										{
											falseSet[d] = 0;
											pinNotchSet[d] = 0;
											plugRot -= setRot;
										}
										else
										{
											
											
											float pinRes = 0.01f;
											
											pinPush[d] -= pinRes;

											if (!clicked)
												obj[currPick].transY -= pinRes;
											else
												rotate(pinRes * -10);
										}
									}
								}
								}
				
				if (pinPush[d] > shearline + pinLen[d] && !((int)plugRot % 360 > -plugPlay + 180 && (int)plugRot % 360 < plugPlay + 180))
				{									
					pinPush[d] = shearline + pinLen[d];
					
				}

				if (pinPush[d] > shearline + pinLen[d] && ((int)plugRot % 360 > -plugPlay + 180 && (int)plugRot % 360 < plugPlay + 180))
				{			
				

					if (pinPush[d] < shearline + setTolerance + pinLen[d])
					{
						
						if (pinOrder[d] == 0 && tension > minTension)
						{
							pinSet[d] = 1;
							pinOverset[d] = 0;				
							plugRot += setRot;

							
							int crap = 0;
							
							int e = 0;
							for(e = 0; e < startTop; e++)
							{
								if (pinSet[e] == 0) crap = 1;							
								pinOrder[e]--;
							}
							
							if (!crap) plugRot = 180.0f + plugPlay + 0.01f;
							
																				
						}					
					}
					else
					{
						if (pinOrder[d] <= 0 && tension > minTension)
						{
							int testme; 

							if (pinSet[d] == 1) 
								testme = rand() % setResistance;
							else
								testme = rand() % oversetResistance;

							if (testme == 0)
							{							
								int pinOrder[10];
								if (pinSet[d] == 1)
								{

									int e = 0;
									for(e = 0; e < startTop; e++)
									{
										pinOrder[e] += startTop;
											pinSet[e] = 0;
											pinOverset[e] = 0;

									}
									
									tension -= setTension;									
								}
								pinOverset[d] = 1;
								pinSet[d] = 0;

								
													
							}
							else
							{
							
							}
						}	
					}	

					if(pinPush[d] > 0.08f + pinLen[d]) 
					{				
						if (!clicked)
                           obj[currPick].transY -= 0.02f;
                        else
                            rotate(0.1f);
                            
                        int eee;
                        
                        for(eee = 0; eee < lock[currLock].numPins; eee++)
                                pinPush[eee] -= 0.02f;
					}
				}
				
				if (pinPush[d] < -0.5) pinPush[d] = -0.5f;

				if (((int)plugRot % 360 > -plugPlay + 180 && (int)plugRot % 360 < plugPlay + 180) )
				{		
					
					
										
					
						
					if (pinSet[d] == 0 )
					{
							
						
					
						if ((obj[c].boundlow + obj[c].transY <= obj[d].boundhigh + obj[d].transY + pinPush[d]) &&
						    (obj[c].boundhigh + obj[c].transY > obj[d].boundhigh + obj[d].transY + pinPush[d])) 
						{
							obj[c].transY = obj[d].boundhigh + obj[d].transY + pinPush[d] - obj[c].boundlow;
							
						}
						else
						{
							if (falseSet[d] == 0)							
							obj[c].transY = obj[d].boundhigh + obj[d].transY + pinPush[d] - obj[c].boundlow;
							
						}
					}

				}
				if (pinPush[d] < -0.5) pinPush[d] = -0.5f;
				
		
			}
			
		}

	    M0 = t;
	}
    }

    return( TRUE );
}

void addContour(float xx, float yy)
{
	pickContour[maxContours].x = xx;
	pickContour[maxContours].y = yy;
	maxContours++;
	
}

void loadContours()
{
	int c = 0;
	
	for(c = obj[currPick+1].firstV; c < obj[currPick+1].numV; c++)
	{
		addContour(v[c].x, v[c].y + 0.4f);

	}

}

void loadLock(int index)
{
	int c;

	/*CLEAR EVERYTHING*/

	plugRot = 180.0f;
	maxContours = 0;
	pickAngle = 0.0f;
	
	
	for(c = 0; c < 10; c++)
	{
		pinPush[c] = 0;
		pinPushing[c] = 0;
		pinPushingContour[c] = 0;
		
		startSprings = 0;
		startTop = 0;
		startPins = 0;
		
		pinSet[c] = 0;
		pinOverset[c] = 0;
		pinNotchSet[c] = 0;
		falseSet[c] = 0;
	}
	
	for(c = 0; c < 500000; c++)
	{
		face[c].type = 0;
		
		if (c < 100)
		{
			obj[c].maxF = 0;
			obj[c].firstV = 0;
			obj[c].numV = 0;
			obj[c].invisible = 0;
		}

	}

	
	maxQuads = 0;
	totalV = 0;
	totalVn = 0;
	totalObj = 0;
	
	/*SET ORDER*/
		
	for(c = 0; c < lock[index].numPins; c++)
	{
		pinOrderPerm[c] = -1;	
	}
	
	for(c = 0; c < lock[index].numPins; c++)
	{
		int d = rand() % lock[index].numPins;
		
		while(pinOrderPerm[d] != -1)
		{
			d = rand() % lock[index].numPins;
		}
		pinOrderPerm[d]	= c;
	}
	
	startPins = 0;
	
	/*LOAD KEY PIN MODELS*/
	
	pinSetResistance = lock[index].tolerance;
	
	for(c = 0; c < lock[index].numPins; c++)
	{
		if (lock[index].pinLen[c] > 0)
		{
			pinLen[c] = lock[index].pinLen[c];
			loadModel("Pin", "pin.obj", 1, 30 + (c * PIN_SPACING), 0, 0, pinLen[c]);	
		}
		else
		{
			break;
		}
	}
	
	/*LOAD DRIVER PIN MODELS*/
	
	startTop = startPins + c;
	
	for(c = 0; c < lock[index].numPins; c++)
	{
		if (lock[index].pinLen[c] > 0)
		{
			pinType[c] = lock[index].topin[c];
			
			if (lock[index].topin[c] == 0)
				loadModel("Normal", "topin.obj", 1, 30 + (c * PIN_SPACING), 5, 0, 0);	
			else if (lock[index].topin[c] == 1)
				loadModel("Spool", "spool.obj", 1, 30 + (c * PIN_SPACING), 5, 0, 0);	
			else if (lock[index].topin[c] == 2)
				loadModel("Serrated", "serrated.obj", 1, 30 + (c * PIN_SPACING), 5, 0, 0);	
			else if (lock[index].topin[c] == 3)
				loadModel("Spoorated", "spoorated.obj", 1, 30 + (c * PIN_SPACING), 5, 0, 0);
		}
		else
		{
			break;
		}
	}
	
	/*LOAD OTHER MODELS*/
	
	obj[startTop + c + 2].invisible = 1;
	obj[startTop + c + 3].invisible = 1;
	
	if (strcmp(visibility, "Plug") == 0)
	{
		obj[startTop + c + 2].invisible = 0;	
	}
	else if (strcmp(visibility, "Realistic") == 0)
	{
		obj[startTop + c + 2].invisible = 0;
		obj[startTop + c + 3].invisible = 0;
	}
	
	loadModel("Tension Wrench", "tension.obj", 1, 29.2, 0, 0, 0);
	loadModel("Tension Arrow", "arrow.obj", 1, 29.5, 0, 0, 0);
	loadModel("Plug", "betterplug.obj", 1, 29.2, 0, 0, 0);
	loadModel("Shell", "shell.obj", 1, 29.2, -1.0, 0, 0);
	
	startSprings = startTop + c + 4;
	
	/*LOAD SPRING MODELS */
	
	for(c = 0; c < lock[index].numPins; c++)
	{
		if (lock[index].pinLen[c] > 0)
		{
				loadModel("Spring", "spring.obj", 0.5, 30 + (c * PIN_SPACING), 1, 0, 0);	
		}
		else
		{
			break;
		}
	}
	
	/*LOAD PICK MODELS */
	
	currPick = startSprings + c;
	
	loadModel("Short Hook", "hook2.obj", 1, 28, 0.0, 0, 0);
	loadModel("Short Hook Contour", "hookcontour.obj", 1, 28, 0.0, 0, 0);
	
	loadModel("DeForest", "deforest.obj", 1, 28, 0.0, 0, 0);
	loadModel("DeForest Contour", "deforestcontour.obj", 1, 28, 0.0, 0, 0);
	
	loadModel("Half-Diamond", "halfdiamond.obj", 1, 28, 0.0, 0, 0);
	loadModel("Half-Diamond Contour", "halfcontour.obj", 1, 28, 0.0, 0, 0);
	
	loadModel("S-Rake", "snake.obj", 1, 28, 0.0, 0, 0);
	loadModel("S-Rake Contour", "snakecontour.obj", 1, 28,  0.0, 0, 0);	
	
	loadModel("Reach", "reachpick.obj", 1, 28, 0.0, 0, 0);
	loadModel("Reach Contour", "reachcontour.obj", 1, 28,  0.0, 0, 0);
	
	loadModel("Long Hook", "long.obj", 1, 28, 0.0, 0, 0);
	loadModel("Long Contour", "longcontour.obj", 1, 28,  0.0, 0, 0);

	loadModel("Bogota Rake", "bogota.obj", 1, 28, 0.0, 0, 0);
	loadModel("Bogota Contour", "bogotacontour.obj", 1, 28,  0.0, 0, 0);	
	
	/*SET COLLISION BOUNDARIES*/
	
	
	for(c = 0; c < startTop + startTop; c++)
	{
		obj[c].boundlow = -0.28f;
		obj[c].boundhigh = 0.33f - pinLen[c];
		obj[c].boundleft = -0.001f;
		obj[c].boundright = 0.001f;		
	
		if (c >= startTop) obj[c].gravity = TRUE;	
	}
}


int main( int argc, char **argv )
{
	pin[0].name = "Normal";
	
	pin[1].name = "Spool";
	pin[1].notchStart[0] = 0.43f;
	pin[1].notchEnd[0] = 0.15f;

	pin[2].name = "Serrated";
	pin[2].notchStart[0] = 0.23f;
	pin[2].notchEnd[0] = 0.16f;
	pin[2].notchStart[1] = 0.35f;
	pin[2].notchEnd[1] = 0.30f;	
	pin[2].notchStart[2] = 0.45f;
	pin[2].notchEnd[2] = 0.40f;	
		
	pin[3].name = "Spoorated";
	pin[3].notchStart[0] = 0.20f;
	pin[3].notchEnd[0] = 0.16f;
	pin[3].notchStart[1] = 0.38f;
	pin[3].notchEnd[1] = 0.23f;	
	pin[3].notchStart[2] = 0.45f;
	pin[3].notchEnd[2] = 0.41f;	
	
	lock[0].name = "Kwikset";
	lock[0].tolerance = 0.1;
	lock[0].fastTimeFull = 16950;
	lock[0].fastTimePlug = 27690;
	lock[0].numPins = 5;
	lock[0].topin[0] = 0;
	lock[0].topin[1] = 0;
	lock[0].topin[2] = 0;
	lock[0].topin[3] = 0;
	lock[0].topin[4] = 0;
		lock[0].pinLen[0] = 0.35f;
		lock[0].pinLen[1] = 0.30f;
		lock[0].pinLen[2] = 0.35f;
		lock[0].pinLen[3] = 0.40f;
		lock[0].pinLen[4] = 0.35f;	
	
	lock[1].name = "Master #3";
	lock[1].tolerance = 0.05;
	lock[1].fastTimeFull = 14510;
	lock[1].fastTimePlug = 23700;
	lock[1].fastTimeRealistic = 54510;
	lock[1].numPins = 4;
	lock[1].topin[0] = 0;
	lock[1].topin[1] = 0;
	lock[1].topin[2] = 0;
	lock[1].topin[3] = 0;
		lock[1].pinLen[0] = 0.25f;
		lock[1].pinLen[1] = 0.30f;
		lock[1].pinLen[2] = 0.25f;
		lock[1].pinLen[3] = 0.35f;
		
	lock[2].name = "Master #140";
	lock[2].tolerance = 0.05;
	lock[2].fastTimeFull = 23630;
	lock[2].fastTimePlug = 36620;
	lock[2].numPins = 4;
	lock[2].topin[0] = 0;
	lock[2].topin[1] = 1;
	lock[2].topin[2] = 0;
	lock[2].topin[3] = 1;
		lock[2].pinLen[0] = 0.20f;
		lock[2].pinLen[1] = 0.35f;
		lock[2].pinLen[2] = 0.10f;
		lock[2].pinLen[3] = 0.25f;
	
	lock[3].name = "Schlage";
	lock[3].tolerance = 0.005;
	lock[3].fastTimeFull = 22950;
	lock[3].fastTimePlug = 35370;
	lock[3].numPins = 5;
	lock[3].topin[0] = 0;
	lock[3].topin[1] = 1;
	lock[3].topin[2] = 0;
	lock[3].topin[3] = 1;
	lock[3].topin[4] = 0;
		lock[3].pinLen[0] = 0.20f;
		lock[3].pinLen[1] = 0.40f;
		lock[3].pinLen[2] = 0.10f;
		lock[3].pinLen[3] = 0.15f;
		lock[3].pinLen[4] = 0.25f;
      
	lock[4].name = "American";
	lock[4].tolerance = 0.001;
	lock[4].fastTimeFull = 33240;
	lock[4].numPins = 6;
	lock[4].topin[0] = 0;
	lock[4].topin[1] = 2;
	lock[4].topin[2] = 3;
	lock[4].topin[3] = 2;
	lock[4].topin[4] = 1;
	lock[4].topin[5] = 2;
		lock[4].pinLen[0] = 0.10f;
		lock[4].pinLen[1] = 0.40f;
		lock[4].pinLen[2] = 0.20f;
		lock[4].pinLen[3] = 0.35f;
		lock[4].pinLen[4] = 0.15f; 
        lock[4].pinLen[5] = 0.40f; 
        
  	lock[5].name = "Be Afraid";
	lock[5].tolerance = 0.0005;
    lock[5].fastTimeFull = 65570;
    lock[5].numPins = 7;
	lock[5].topin[0] = 3;
	lock[5].topin[1] = 2;
	lock[5].topin[2] = 3;
	lock[5].topin[3] = 2;
	lock[5].topin[4] = 3;
	lock[5].topin[5] = 2;
	lock[5].topin[6] = 3;
		lock[5].pinLen[0] = 0.10f;
		lock[5].pinLen[1] = 0.45f;
		lock[5].pinLen[2] = 0.10f;
		lock[5].pinLen[3] = 0.45f;
		lock[5].pinLen[4] = 0.10f; 
        lock[5].pinLen[5] = 0.45f;
        lock[5].pinLen[6] = 0.10f;

	/*Pin lengths are in increments of 0.05f between 0.05f(long) and 0.45f(long) */
	
	/*notchEnd defines bottom of notch (larger = lower)*/
	






    /* Flags to pass to SDL_SetVideoMode */
    int videoFlags;
    /* main loop variable */
    int done = FALSE;
    /* used to collect events */
    SDL_Event event;
    /* this holds some info about our display */
    const SDL_VideoInfo *videoInfo;
    /* whether or not the window is active */
    int isActive = TRUE;

    /* initialize SDL */
    if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
	    fprintf( stderr, "Video initialization failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* Fetch the video info */
    videoInfo = SDL_GetVideoInfo( );

    if ( !videoInfo )
	{
	    fprintf( stderr, "Video query failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* the flags to pass to SDL_SetVideoMode */
    videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL */
    videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering */
    videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
   	videoFlags |= SDL_FULLSCREEN;
	/*  videoFlags |= SDL_RESIZABLE;       Enable window resizing */

    /* This checks to see if surfaces can be stored in memory */
    if ( videoInfo->hw_available )
	videoFlags |= SDL_HWSURFACE;
    else
	videoFlags |= SDL_SWSURFACE;

    /* This checks if hardware blits can be done */
    if ( videoInfo->blit_hw )
	videoFlags |= SDL_HWACCEL;

    /* Sets up OpenGL double buffering */
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

    /* get a SDL surface */
    surface = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,
				videoFlags );

	SDL_ShowCursor(GL_FALSE);
	
	
	/*SDL_WM_ToggleFullScreen(surface);*/
	
    /* Verify there is a surface */
    if ( !surface )
	{
	    fprintf( stderr,  "Video mode set failed: %s\n", SDL_GetError( ) );
	    Quit( 1 );
	}

    /* initialize OpenGL */
    initGL( );

    /* resize the initial window */
    resizeWindow( SCREEN_WIDTH, SCREEN_HEIGHT );

    /* wait for events */
    while ( !done )
	{

	    /* handle the events in the queue */

	    while ( SDL_PollEvent( &event ) )
		{
		    switch( event.type )
			{

			case SDL_ACTIVEEVENT:
			    /* Something's happend with our focus
			     * If we lost focus or we are iconified, we
			     * shouldn't draw the screen
			     */
			    if ( event.active.gain == 0 )
					isActive = FALSE;
			    else
					isActive = TRUE;
				
			    break;			    
			case SDL_VIDEORESIZE:
			    /* handle resize event */
			    surface = SDL_SetVideoMode( event.resize.w,
							event.resize.h,
							16, videoFlags );

			    if ( !surface )
				{
				    fprintf( stderr, "Could not get a surface after resize: %s\n", SDL_GetError( ) );
				    Quit( 1 );
				}
			    resizeWindow( event.resize.w, event.resize.h );
			    break;
			case SDL_KEYDOWN:
			    /* handle key presses */
			    handleKeyPress( &event.key.keysym, 1 );
				
			    break;
			case SDL_KEYUP:
				handleKeyPress( &event.key.keysym, 0);
				break;
			case SDL_MOUSEBUTTONDOWN:
				if (SDL_BUTTON_LEFT) clicked = 1;
				break;
			case SDL_MOUSEBUTTONUP:
				if (SDL_BUTTON_LEFT) clicked = 0;
				break;
			case SDL_MOUSEMOTION:
				
				handleMouse(event.motion.xrel, event.motion.yrel, event.motion.x, event.motion.y, event.motion.state);
				
				break;
			case SDL_QUIT:
			    /* handle quit requests */
			    done = TRUE;
			    break;
			default:
			    break;
			}
		}

	    /* draw the scene */
	    if ( isActive )
			drawGLScene( );
	}

    /* clean ourselves up and exit */
    Quit( 0 );

    /* Should never get here */
    return( 0 );
}
